SWOT Report Scope: {{ swot_analysis.company_listing.display_name }} ({{ swot_analysis.company_listing.country }}) - Internal Knowledge
{%- if swot_analysis.use_earnings_call or swot_analysis.use_web_sources %} with {% endif -%}
{%- if swot_analysis.use_earnings_call %}Earnings Calls(from {{ swot_analysis.earnings_call_start_date }}){% endif -%}
{%- if swot_analysis.use_earnings_call and swot_analysis.use_web_sources %} and {% endif -%}
{%- if swot_analysis.use_web_sources %}Web Search{% endif -%}